"""
PDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

    Maintain and explore the contents of nested collections.

"""
import asyncio
import time

# import json
from pyegeria._client import Client
from pyegeria._exceptions import (InvalidParameterException, )
from pyegeria._validators import (validate_guid, validate_search_string, )
from pyegeria.utils import body_slimmer


class CollectionManager(Client):
    """
    Maintain and explore the contents of nested collections. These collections can be used to represent digital
    products, or collections of resources for a particular project or team. They can be used to organize assets and
    other resources into logical groups.

    Attributes:

        server_name: str
            The name of the View Server to connect to.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None

     """

    def __init__(self, server_name: str, platform_url: str, token: str = None, user_id: str = None,
                 user_pwd: str = None, sync_mode: bool = True):
        self.command_base: str = f"/api/open-metadata/collection-manager/collections"
        Client.__init__(self, server_name, platform_url, user_id=user_id, token=token, async_mode=sync_mode)

    #
    #       Retrieving Collections - https://egeria-project.org/concepts/collection
    #
    async def _async_get_linked_collections(self, parent_guid: str, server_name: str = None, start_from: int = 0,
                                            page_size: int = None) -> list:
        """  Returns the list of collections that are linked off of the supplied element. Async version.

        Parameters
        ----------
        parent_guid: str
            The identity of the parent to find linked collections from.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List

        A list of collections linked off of the supplied element.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        if page_size is None:
            page_size = self.page_size

        body = {

        }

        url = (f"{self.platform_url}/servers/{server_name}/api/open-metadata/collection-manager/"
               f"metadata-elements/{parent_guid}/collections?startFrom={start_from}&pageSize={page_size}")

        resp = await self._async_make_request("POST", url, body)
        return resp.json()

    def get_linked_collections(self, parent_guid: str, server_name: str = None, start_from: int = 0,
                               page_size: int = None) -> list:
        """  Returns the list of collections that are linked off of the supplied element.

        Parameters
        ----------
        parent_guid: str
            The identity of the parent to find linked collections from.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List

        A list of collections linked off of the supplied element.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_linked_collections(parent_guid, server_name, start_from, page_size))
        return resp

    async def _async_get_classified_collections(self, classification: str, server_name: str = None, start_from: int = 0,
                                                page_size: int = None) -> list | str:
        """ Returns the list of collections with a particular classification.  These classifications
            are typically "RootCollection", "Folder" or "DigitalProduct". Async version.

        Parameters
        ----------
        classification: str
            The classification of the collection to inspect.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.

        Returns
        -------
        list | str
            The list of collections (if found) with the specified classification. Returns a string if none found.

        Raises
        ------
         InvalidParameterException
             If the client passes incorrect parameters on the request - such as bad URLs or invalid values.
         PropertyServerException
             Raised by the server when an issue arises in processing a valid request.
         NotAuthorizedException
             The principle specified by the user_id does not have authorization for the requested action.
        Notes
        -----
        """
        if server_name is None:
            server_name = self.server_name
        if page_size is None:
            page_size = self.page_size

        body = {"filter": classification}

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/by-classifications?"
               f"startFrom={start_from}&pageSize={page_size}")

        resp = await self._async_make_request("POST", url, body)
        # result = resp.json().get("elements","No elements found")
        result = resp.json().get("elements", "No Elements to return")
        return result

    def get_classified_collections(self, classification: str, server_name: str = None, start_from: int = 0,
                                   page_size: int = None) -> list | str:
        """  Returns the list of collections with a particular classification.  These classifications
             are typically "RootCollection", "Folder" or "DigitalProduct".

        Parameters
        ----------
        classification: str
            The classification of the collection to inspect.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List | str

        A list of collections linked off of the supplied element.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_classified_collections(classification, server_name, start_from, page_size))
        return resp

    async def _async_find_collections(self, search_string: str, effective_time: str = None, starts_with: bool = False,
                                      ends_with: bool = False, ignore_case: bool = False, server_name: str = None,
                                      start_from: int = 0, page_size: int = None) -> list | str:
        """ Returns the list of collections matching the search string.
            The search string is located in the request body and is interpreted as a plain string.
            The request parameters, startsWith, endsWith and ignoreCase can be used to allow a fuzzy search.

        Parameters
        ----------
        search_string: str,
            Search string to use to find matching collections. If the search string is '*' then all glossaries returned.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time. ISO8601 format is assumed.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        starts_with : bool, [default=False], optional
            Starts with the supplied string.
        ends_with : bool, [default=False], optional
            Ends with the supplied string
        ignore_case : bool, [default=False], optional
            Ignore case when searching
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List | str

        A list of collections match matching the search string. Returns a string if none found.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        if page_size is None:
            page_size = self.page_size
        starts_with_s = str(starts_with).lower()
        ends_with_s = str(ends_with).lower()
        ignore_case_s = str(ignore_case).lower()

        validate_search_string(search_string)

        if search_string == '*':
            search_string = None

        body = {"filter": search_string, "effective_time": effective_time}

        body_s = body_slimmer(body)
        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/"
               f"by-search-string?startFrom={start_from}&pageSize={page_size}&startsWith={starts_with_s}&"
               f"endsWith={ends_with_s}&ignoreCase={ignore_case_s}")

        resp = await self._async_make_request("POST", url, body_s)
        return resp.json().get("elements", "No elements found")

    def find_collections(self, search_string: str, effective_time: str = None, starts_with: bool = False,
                         ends_with: bool = False, ignore_case: bool = False, server_name: str = None,
                         start_from: int = 0, page_size: int = None) -> list | str:
        """ Returns the list of collections matching the search string. Async version.
            The search string is located in the request body and is interpreted as a plain string.
            The request parameters, startsWith, endsWith and ignoreCase can be used to allow a fuzzy search.

        Parameters
        ----------
        search_string: str,
            Search string to use to find matching collections. If the search string is '*' then all glossaries returned.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time. Time in ISO8601 format is assumed.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        starts_with : bool, [default=False], optional
            Starts with the supplied string.
        ends_with : bool, [default=False], optional
            Ends with the supplied string
        ignore_case : bool, [default=False], optional
            Ignore case when searching
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List | str

        A list of collections match matching the search string. Returns a string if none found.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_find_collections(search_string, effective_time, starts_with, ends_with, ignore_case,
                                         server_name, start_from, page_size))

        return resp

    async def _async_get_collections_by_name(self, name: str, effective_time: str = None, server_name: str = None,
                                             start_from: int = 0, page_size: int = None) -> list | str:
        """ Returns the list of collections with a particular name.

        Parameters
        ----------
        name: str,
            name to use to find matching collections.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time. Time in ISO8601 format is assumed.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List | str

        A list of collections match matching the name. Returns a string if none found.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        if page_size is None:
            page_size = self.page_size

        validate_search_string(name)

        body = {"filter": name, effective_time: effective_time, }
        body_s = body_slimmer(body)
        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/"
               f"by-name?startFrom={start_from}&pageSize={page_size}")

        resp = await self._async_make_request("POST", url, body_s)
        return resp.json().get("elements", "No elements found")

    def get_collections_by_name(self, name: str, effective_time: str = None, server_name: str = None,
                                start_from: int = 0, page_size: int = None) -> list | str:
        """ Returns the list of collections matching the search string. Async version.
            The search string is located in the request body and is interpreted as a plain string.
            The request parameters, startsWith, endsWith and ignoreCase can be used to allow a fuzzy search.

        Parameters
        ----------
        name: str,
            name to use to find matching collections.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time. Time in ISO8601 format is assumed.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.

        Returns
        -------
        List | str

        A list of collections match matching the search string. Returns a string if none found.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_collections_by_name(name, effective_time, server_name, start_from, page_size))

        return resp

    async def _async_get_collections_by_type(self, collection_type: str, effective_time: str = None,
                                             server_name: str = None, start_from: int = 0,
                                             page_size: int = None) -> list | str:
        """ Returns the list of collections with a particular collectionType. This is an optional text field in the
            collection element.

        Parameters
        ----------
        collection_type: str,
            collection_type to use to find matching collections.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time. Time in ISO8601 format is assumed.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.

        Returns
        -------
        List | str

        A list of collections match matching the name. Returns a string if none found.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        if page_size is None:
            page_size = self.page_size

        validate_search_string(collection_type)

        body = {"filter": collection_type, effective_time: effective_time, }
        body_s = body_slimmer(body)

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/"
               f"by-collection-type?startFrom={start_from}&pageSize={page_size}")

        resp = await self._async_make_request("POST", url, body_s)
        return resp.json().get("elements", "No elements found")

    def get_collections_by_type(self, collection_type: str, effective_time: str = None, server_name: str = None,
                                start_from: int = 0, page_size: int = None) -> list | str:
        """ Returns the list of collections matching the search string. Async version.
            The search string is located in the request body and is interpreted as a plain string.
            The request parameters, startsWith, endsWith and ignoreCase can be used to allow a fuzzy search.

        Parameters
        ----------
        collection_type: str,
            collection type to find.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time. Time in ISO8601 format is assumed.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.

        Returns
        -------
        List | str

        A list of collections match matching the search string. Returns a string if none found.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_collections_by_type(collection_type, effective_time, server_name, start_from, page_size))

        return resp

    async def _async_get_collection(self, collection_guid: str, effective_time: str = None,
                                    server_name: str = None) -> dict | str:
        """ Return the properties of a specific collection. Async version.

        Parameters
        ----------
        collection_guid: str,
            unique identifier of the collection.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time. Time in ISO8601 format is assumed.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        dict | str

        A JSON dict representing the specified collection. Returns a string if none found.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        validate_guid(collection_guid)

        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/{collection_guid}"
        body = {"effective_time": effective_time, }
        resp = await self._async_make_request("GET", url, body)
        return resp.json()

    def get_collection(self, collection_guid: str, effective_time: str = None, server_name: str = None) -> dict | str:
        """ Return the properties of a specific collection.

        Parameters
        ----------
        collection_guid: str,
            unique identifier of the collection.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        dict | str

        A JSON dict representing the specified collection. Returns a string if none found.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(self._async_get_collection(collection_guid, effective_time, server_name))

        return resp

    #
    #   Create collection methods
    #
    async def _async_create_collection_w_body(self, classification_name: str, body: dict,
                                              server_name: str = None) -> str:
        """  Create Collections: https://egeria-project.org/concepts/collection Async version.

            Parameters
            ----------
            classification_name: str
                Type of collection to create; e.g RootCollection, Folder, Set, DigitalProduct, etc.
            body: dict
                A dict representing the details of the collection to create.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated with the
                instance is used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

    """
        if server_name is None:
            server_name = self.server_name

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}?"
               f"classificationName={classification_name}")

        resp = await self._async_make_request("POST", url, body)
        return resp.json().get("guid", "No GUID returned")

    def create_collection_w_body(self, classification_name: str, body: dict, server_name: str = None) -> str:
        """ Create Collections: https://egeria-project.org/concepts/collection

            Parameters
            ----------
            classification_name: str
                Type of collection to create; e.g RootCollection, Folder, Set, DigitalProduct, etc.
            body: dict
                A dict representing the details of the collection to create.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated with the instance is
                used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(self._async_create_collection_w_body(classification_name, body, server_name))
        return resp

    async def _async_create_collection(self, classification_name: str, anchor_guid: str, parent_guid: str,
                                       parent_relationship_type_name: str, parent_at_end1: bool, display_name: str,
                                       description: str, collection_type: str, is_own_anchor: bool = False,
                                       collection_ordering: str = None, order_property_name: str = None,
                                       server_name: str = None) -> str:
        """ Create Collections: https://egeria-project.org/concepts/collection Async version.

            Parameters
            ----------
            classification_name: str
                Type of collection to create; e.g RootCollection, Folder, Set, DigitalProduct, etc.
            anchor_guid: str
                The unique identifier of the element that should be the anchor for the new element. Set to null if no
                anchor, or if this collection is to be its own anchor.
            parent_guid: str
               The optional unique identifier for an element that should be connected to the newly created element.
               If this property is specified, parentRelationshipTypeName must also be specified
            parent_relationship_type_name: str
                The name of the relationship, if any, that should be established between the new element and the parent
                element. Examples could be "ResourceList" or "DigitalServiceProduct".
            parent_at_end1: bool
                Identifies which end any parent entity sits on the relationship.
            display_name: str
                The display name of the element. Will also be used as the basis of the qualified_name.
            description: str
                A description of the collection.
            collection_type: str
                Add appropriate valid value for the collection type.
            is_own_anchor: bool, optional, defaults to False
                Indicates if the collection should classified as its own anchor or not.
            collection_ordering: str, optional, defaults to "OTHER"
                Specifies the sequencing to use in a collection. Examples include "NAME", "OWNER", "DATE_CREATED",
                 "OTHER"
            order_property_name: str, optional, defaults to "Something"
                Property to use for sequencing if collection_ordering is "OTHER"
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated with the instance is
                 used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            """
        if server_name is None:
            server_name = self.server_name

        if parent_guid is None:
            is_own_anchor = False
        is_own_anchor_s = str(is_own_anchor).lower()

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}?"
               f"classificationName={classification_name}")

        body = {"anchorGUID": anchor_guid, "isOwnAnchor": is_own_anchor_s, "parentGUID": parent_guid,
                "parentRelationshipTypeName": parent_relationship_type_name, "parentAtEnd1": parent_at_end1,
                "collectionProperties": {"class": "CollectionProperties",
                                         "qualifiedName": f"{classification_name}-{display_name}-{time.asctime()}",
                                         "name": display_name,
                                         "description": description, "collectionType": collection_type,
                                         "collectionOrdering": collection_ordering,
                                         "orderPropertyName": order_property_name}}

        resp = await self._async_make_request("POST", url, body)
        return resp.json().get("guid", "No GUID returned")

    def create_collection(self, classification_name: str, anchor_guid: str, parent_guid: str,
                          parent_relationship_type_name: str, parent_at_end1: bool, display_name: str, description: str,
                          collection_type: str, is_own_anchor: bool = False, collection_ordering: str = "OTHER",
                          order_property_name: str = "Something", server_name: str = None) -> str:
        """  Create Collections: https://egeria-project.org/concepts/collection

            Parameters
            ----------
            classification_name: str
                Type of collection to create; e.g RootCollection, Folder, Set, DigitalProduct, etc.
            anchor_guid: str
                The unique identifier of the element that should be the anchor for the new element.
                Set to null if no anchor, or if this collection is to be its own anchor.
            parent_guid: str
               The optional unique identifier for an element that should be connected to the newly created element.
               If this property is specified, parentRelationshipTypeName must also be specified
            parent_relationship_type_name: str
                The name of the relationship, if any, that should be established between the new element and the parent
                element. Examples could be "ResourceList" or "DigitalServiceProduct".
            parent_at_end1: bool
                Identifies which end any parent entity sits on the relationship.
            display_name: str
                The display name of the element. Will also be used as the basis of the qualified_name.
            description: str
                A description of the collection.
            collection_type: str
                Add appropriate valid value for the collection type.
            is_own_anchor: bool, optional, defaults to False
                Indicates if the collection should classified as its own anchor or not.
            collection_ordering: str, optional, defaults to "OTHER"
                Specifies the sequencing to use in a collection. Examples include "NAME", "OWNER",
                "DATE_CREATED", "OTHER"
            order_property_name: str, optional, defaults to "Something"
                Property to use for sequencing if collection_ordering is "OTHER"
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated with the
                instance is used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_create_collection(classification_name, anchor_guid, parent_guid, parent_relationship_type_name,
                                          parent_at_end1, display_name, description, collection_type, is_own_anchor,
                                          collection_ordering, order_property_name, server_name))
        return resp

    async def _async_create_root_collection(self, anchor_guid: str, parent_guid: str,
                                            parent_relationship_type_name: str, parent_at_end1: bool, display_name: str,
                                            description: str, collection_type: str, is_own_anchor: bool = False,
                                            server_name: str = None) -> str:
        """ Create a new collection with the RootCollection classification.  Used to identify the top of a
            collection hierarchy. Async version.

            Parameters
            ----------
            anchor_guid: str
                The unique identifier of the element that should be the anchor for the new element.
                Set to null if no anchor, or if this collection is to be its own anchor.
            parent_guid: str
               The optional unique identifier for an element that should be connected to the newly created element.
               If this property is specified, parentRelationshipTypeName must also be specified
            parent_relationship_type_name: str
                The name of the relationship, if any, that should be established between the new element and the parent
                element. Examples could be "ResourceList" or "DigitalServiceProduct".
            parent_at_end1: bool
                Identifies which end any parent entity sits on the relationship.
            display_name: str
                The display name of the element. Will also be used as the basis of the qualified_name.
            description: str
                A description of the collection.
            collection_type: str
                Add appropriate valid value for the collection type.
            is_own_anchor: bool, optional, defaults to False
                Indicates if the collection should classified as its own anchor or not.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If none, the server name associated with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            """
        if server_name is None:
            server_name = self.server_name
        is_own_anchor_s = str(is_own_anchor).lower()
        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/root-collection"

        body = {"anchorGUID": anchor_guid, "isOwnAnchor": is_own_anchor_s, "parentGUID": parent_guid,
                "parentRelationshipTypeName": parent_relationship_type_name, "parentAtEnd1": parent_at_end1,
                "collectionProperties": {"class": "CollectionProperties",
                                         "qualifiedName": f"root-collection-{display_name}-{time.asctime()}",
                                         "name": display_name,
                                         "description": description, "collectionType": collection_type,

                                         }}

        resp = await self._async_make_request("POST", url, body)
        return resp.json().get("guid", "No GUID Returned")

    def create_root_collection(self, anchor_guid: str, parent_guid: str, parent_relationship_type_name: str,
                               parent_at_end1: bool, display_name: str, description: str, collection_type: str,
                               is_own_anchor: bool = False, server_name: str = None) -> str:
        """  Create a new collection with the RootCollection classification.  Used to identify the top of a
             collection hierarchy.

            Parameters
            ----------
            anchor_guid: str
                The unique identifier of the element that should be the anchor for the new element.
                Set to null if no anchor,
                or if this collection is to be its own anchor.
            parent_guid: str
               The optional unique identifier for an element that should be connected to the newly created element.
               If this property is specified, parentRelationshipTypeName must also be specified
            parent_relationship_type_name: str
                The name of the relationship, if any, that should be established between the new element and the parent
                element. Examples could be "ResourceList" or "DigitalServiceProduct".
            parent_at_end1: bool
                Identifies which end any parent entity sits on the relationship.
            display_name: str
                The display name of the element. Will also be used as the basis of the qualified_name.
            description: str
                A description of the collection.
            collection_type: str
                Add appropriate valid value for the collection type.
            is_own_anchor: bool, optional, defaults to False
                Indicates if the collection should classified as its own anchor or not.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If None, the server name associated with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_create_root_collection(anchor_guid, parent_guid, parent_relationship_type_name, parent_at_end1,
                                               display_name, description, collection_type, is_own_anchor, server_name))
        return resp

    async def _async_create_data_spec_collection(self, anchor_guid: str, parent_guid: str,
                                                 parent_relationship_type_name: str, parent_at_end1: bool,
                                                 display_name: str, description: str, collection_type: str,
                                                 is_own_anchor: bool = True, collection_ordering: str = "OTHER",
                                                 order_property_name: str = "Something",
                                                 server_name: str = None) -> str:
        """  Create a new collection with the DataSpec classification.  Used to identify a collection of data fields
             and schema types. Async version.

            Parameters
            ----------
            anchor_guid: str
                The unique identifier of the element that should be the anchor for the new element.
                Set to null if no anchor, or if this collection is to be its own anchor.
            parent_guid: str
               The optional unique identifier for an element that should be connected to the newly created element.
               If this property is specified, parentRelationshipTypeName must also be specified
            parent_relationship_type_name: str
                The name of the relationship, if any, that should be established between the new element and the parent
                element. Examples could be "ResourceList" or "DigitalServiceProduct".
            parent_at_end1: bool
                Identifies which end any parent entity sits on the relationship.
            display_name: str
                The display name of the element. Will also be used as the basis of the qualified_name.
            description: str
                A description of the collection.
            collection_type: str
                Add appropriate valid value for the collection type.
            is_own_anchor: bool, optional, defaults to False
                Indicates if the collection should classified as its own anchor or not.
            collection_ordering: str, optional, defaults to "OTHER"
                Specifies the sequencing to use in a collection. Examples include "NAME", "OWNER",
                "DATE_CREATED", "OTHER"
            order_property_name: str, optional, defaults to "Something"
                Property to use for sequencing if collection_ordering is "OTHER"
            server_name: str, optional, defaults to None
                The name of the server to  configure. If None, the server name associated with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action
        """

        if server_name is None:
            server_name = self.server_name
        is_own_anchor_s = str(is_own_anchor).lower()
        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/data-spec-collection"

        body = {"anchorGUID": anchor_guid, "isOwnAnchor": is_own_anchor_s, "parentGUID": parent_guid,
                "parentRelationshipTypeName": parent_relationship_type_name, "parentAtEnd1": parent_at_end1,
                "collectionProperties": {"class": "CollectionProperties",
                                         "qualifiedName": f"data-spec-collection-{display_name}-{time.asctime()}",
                                         "name": display_name,
                                         "description": description, "collectionType": collection_type,
                                         "collectionOrdering": collection_ordering,
                                         "orderPropertyName": order_property_name}}

        resp = await self._async_make_request("POST", url, body)
        return resp.json().get("guid", "No GUID Returned")

    def create_data_spec_collection(self, anchor_guid: str, parent_guid: str, parent_relationship_type_name: str,
                                    parent_at_end1: bool, display_name: str, description: str, collection_type: str,
                                    is_own_anchor: bool, collection_ordering: str = "OTHER",
                                    order_property_name: str = "Something", server_name: str = None) -> str:
        """  Create a new collection with the DataSpec classification.  Used to identify a collection of data fields
         and schema types.

        Parameters
        ----------
        anchor_guid: str
            The unique identifier of the element that should be the anchor for the new element.
            Set to null if no anchor, or if this collection is to be its own anchor.
        parent_guid: str
           The optional unique identifier for an element that should be connected to the newly created element.
           If this property is specified, parentRelationshipTypeName must also be specified
        parent_relationship_type_name: str
            The name of the relationship, if any, that should be established between the new element and the parent
            element. Examples could be "ResourceList" or "DigitalServiceProduct".
        parent_at_end1: bool
            Identifies which end any parent entity sits on the relationship.
        display_name: str
            The display name of the element. Will also be used as the basis of the qualified_name.
        description: str
            A description of the collection.
        collection_type: str
            Add appropriate valid value for the collection type.
        is_own_anchor: bool, optional, defaults to False
            Indicates if the collection should classified as its own anchor or not.
        collection_ordering: str, optional, defaults to "OTHER"
            Specifies the sequencing to use in a collection. Examples include "NAME", "OWNER", "DATE_CREATED", "OTHER"
        order_property_name: str, optional, defaults to "Something"
            Property to use for sequencing if collection_ordering is "OTHER"
        server_name: str, optional, defaults to None
            The name of the server to  configure. If not provided, the server name associated with the instance is used.

        Returns
        -------
        str - the guid of the created collection

        A JSON dict representing the specified collection. Returns a string if none found.

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_create_data_spec_collection(anchor_guid, parent_guid, parent_relationship_type_name,
                                                    parent_at_end1, display_name, description, collection_type,
                                                    is_own_anchor, collection_ordering, order_property_name,
                                                    server_name))
        return resp

    async def _async_create_folder_collection(self, anchor_guid: str, parent_guid: str,
                                              parent_relationship_type_name: str, parent_at_end1: bool,
                                              display_name: str, description: str, collection_type: str,
                                              is_own_anchor: bool = True, collection_ordering: str = "OTHER",
                                              order_property_name: str = "Something", server_name: str = None) -> str:
        """ Create a new collection with the Folder classification.  This is used to identify the organizing
            collections in a collection hierarchy. Async version.

            Parameters
            ----------
            anchor_guid: str
                The unique identifier of the element that should be the anchor for the new element.
                Set to null if no anchor, or if this collection is to be its own anchor.
            parent_guid: str
               The optional unique identifier for an element that should be connected to the newly created element.
               If this property is specified, parentRelationshipTypeName must also be specified
            parent_relationship_type_name: str
                The name of the relationship, if any, that should be established between the new element and the parent
                element. Examples could be "ResourceList" or "DigitalServiceProduct".
            parent_at_end1: bool
                Identifies which end any parent entity sits on the relationship.
            display_name: str
                The display name of the element. Will also be used as the basis of the qualified_name.
            description: str
                A description of the collection.
            collection_type: str
                Add appropriate valid value for the collection type.
            is_own_anchor: bool, optional, defaults to False
                Indicates if the collection should classified as its own anchor or not.
            collection_ordering: str, optional, defaults to "OTHER"
                Specifies the sequencing to use in a collection. Examples include "NAME", "OWNER",
                "DATE_CREATED", "OTHER"
            order_property_name: str, optional, defaults to "Something"
                Property to use for sequencing if collection_ordering is "OTHER"
            server_name: str, optional, defaults to None
                The name of the server to  configure. If None, the server name associated with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        is_own_anchor_s = str(is_own_anchor).lower()

        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/folder"

        body = {"anchorGUID": anchor_guid, "isOwnAnchor": is_own_anchor_s, "parentGUID": parent_guid,
                "parentRelationshipTypeName": parent_relationship_type_name, "parentAtEnd1": parent_at_end1,
                "collectionProperties": {"class": "CollectionProperties",
                                         "qualifiedName": f"folder-collection-{display_name}-{time.asctime()}",
                                         "name": display_name,
                                         "description": description, "collectionType": collection_type,
                                         "collectionOrdering": collection_ordering,
                                         "orderPropertyName": order_property_name}}

        resp = await self._async_make_request("POST", url, body)
        return resp.json().get("guid", "No GUID returned")

    def create_folder_collection(self, anchor_guid: str, parent_guid: str, parent_relationship_type_name: str,
                                 parent_at_end1: bool, display_name: str, description: str, collection_type: str,
                                 is_own_anchor: bool, collection_ordering: str = "OTHER",
                                 order_property_name: str = "Something", server_name: str = None) -> str:
        """ Create a new collection with the Folder classification.  This is used to identify the organizing
            collections in a collection hierarchy.

            Parameters
            ----------
            anchor_guid: str
                The unique identifier of the element that should be the anchor for the new element.
                Set to null if no anchor, or if this collection is to be its own anchor.
            parent_guid: str
               The optional unique identifier for an element that should be connected to the newly created element.
               If this property is specified, parentRelationshipTypeName must also be specified
            parent_relationship_type_name: str
                The name of the relationship, if any, that should be established between the new element and the parent
                element. Examples could be "ResourceList" or "DigitalServiceProduct".
            parent_at_end1: bool
                Identifies which end any parent entity sits on the relationship.
            display_name: str
                The display name of the element. Will also be used as the basis of the qualified_name.
            description: str
                A description of the collection.
            collection_type: str
                Add appropriate valid value for the collection type.
            is_own_anchor: bool, optional, defaults to False
                Indicates if the collection should classified as its own anchor or not.
            collection_ordering: str, optional, defaults to "OTHER"
                Specifies the sequencing to use in a collection. Examples include "NAME", "OWNER", "DATE_CREATED",
                "OTHER"
            order_property_name: str, optional, defaults to "Something"
                Property to use for sequencing if collection_ordering is "OTHER"
            server_name: str, optional, defaults to None
                The name of the server to  configure. If None, the server name associated with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            A JSON dict representing the specified collection. Returns a string if none found.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_create_folder_collection(anchor_guid, parent_guid, parent_relationship_type_name,
                                                 parent_at_end1, display_name, description, collection_type,
                                                 is_own_anchor, collection_ordering, order_property_name, server_name))
        return resp

    async def _async_create_collection_from_template(self, body: dict, server_name: str = None) -> str:
        """ Create a new metadata element to represent a collection using an existing metadata element as a template.
            The template defines additional classifications and relationships that are added to the new collection.
            Async version.

            Parameters
            ----------

            body: dict
                A dict representing the details of the collection to create.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If None, the server name associated with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            JSON Structure looks like:

            {
              "class": "TemplateRequestBody",
              "anchorGUID": "anchor GUID, if set then isOwnAnchor=false",
              "isOwnAnchor": false,
              "parentGUID": "parent GUID, if set, set all parameters beginning 'parent'",
              "parentRelationshipTypeName": "open metadata type name",
              "parentAtEnd1": true,
              "templateGUID": "template GUID",
              "replacementProperties": {
                "class": "ElementProperties",
                "propertyValueMap" : {
                  "propertyName" : {
                    "class": "PrimitiveTypePropertyValue",
                    "typeName": "string",
                    "primitiveTypeCategory" : "OM_PRIMITIVE_TYPE_STRING",
                    "primitiveValue" : "value of property"
                  }
                }
              },
              "placeholderPropertyValues" : {
                "placeholderProperty1Name" : "property1Value",
                "placeholderProperty2Name" : "property2Value"
              }
            }

    """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/from-template"

        resp = await self._async_make_request("POST", url, body)
        return resp.json().get("guid", "No GUID Returned")

    def create_collection_from_template(self, body: dict, server_name: str = None) -> str:
        """ Create a new metadata element to represent a collection using an existing metadata element as a template.
            The template defines additional classifications and relationships that are added to the new collection.

            Parameters
            ----------
            body: dict
                A dict representing the details of the collection to create.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If None, the server name associated with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            JSON Structure looks like:

            {
              "class": "TemplateRequestBody",
              "anchorGUID": "anchor GUID, if set then isOwnAnchor=false",
              "isOwnAnchor": false,
              "parentGUID": "parent GUID, if set, set all parameters beginning 'parent'",
              "parentRelationshipTypeName": "open metadata type name",
              "parentAtEnd1": true,
              "templateGUID": "template GUID",
              "replacementProperties": {
                "class": "ElementProperties",
                "propertyValueMap" : {
                  "propertyName" : {
                    "class": "PrimitiveTypePropertyValue",
                    "typeName": "string",
                    "primitiveTypeCategory" : "OM_PRIMITIVE_TYPE_STRING",
                    "primitiveValue" : "value of property"
                  }
                }
              },
              "placeholderPropertyValues" : {
                "placeholderProperty1Name" : "property1Value",
                "placeholderProperty2Name" : "property2Value"
              }
            }
        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(self._async_create_collection_from_template(body, server_name))
        return resp

    async def _async_create_digital_product(self, body: dict, server_name: str = None) -> str:
        """ Create a new collection that represents a digital product. Async version.

            Parameters
            ----------
            body: dict
                A dict representing the details of the collection to create.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated
                with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            JSON Structure looks like:
            {
              "class" : "NewDigitalProductRequestBody",
              "anchorGUID" : "anchor GUID, if set then isOwnAnchor=false",
              "isOwnAnchor" : false,
              "parentGUID" : "parent GUID, if set, set all parameters beginning 'parent'",
              "parentRelationshipTypeName" : "open metadata type name",
              "parentAtEnd1": true,
              "collectionProperties": {
                "class" : "CollectionProperties",
                "qualifiedName": "Must provide a unique name here",
                "name" : "Add display name here",
                "description" : "Add description of the collection here",
                "collectionType": "Add appropriate valid value for type",
                "collectionOrdering" : "OTHER",
                "orderPropertyName" : "Add property name if 'collectionOrdering' is OTHER"
              },
              "digitalProductProperties" : {
                "class" : "DigitalProductProperties",
                "productStatus" : "ACTIVE",
                "productName" : "Add name here",
                "productType" : "Add valid value here",
                "description" : "Add description here",
                "introductionDate" : "date",
                "maturity" : "Add valid value here",
                "serviceLife" : "Add the estimated lifetime of the product",
                "currentVersion": "V1.0",
                "nextVersion": "V1.1",
                "withdrawDate": "date",
                "additionalProperties": {
                  "property1Name" : "property1Value",
                  "property2Name" : "property2Value"
                }
              }
            }
    """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.platform_url}/servers/{server_name}/api/open-metadata/collection-manager/digital-products"

        resp = await self._async_make_request("POST", url, body)
        return resp.json().get("guid", "No GUID returned")

    def create_digital_product(self, body: dict, server_name: str = None) -> str:
        """ Create a new collection that represents a digital product. Async version.

            Parameters
            ----------
            body: dict
                A dict representing the details of the collection to create.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated
                with the instance is used.

            Returns
            -------
            str - the guid of the created collection

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            JSON Structure looks like:
            {
              "class" : "NewDigitalProductRequestBody",
              "anchorGUID" : "anchor GUID, if set then isOwnAnchor=false",
              "isOwnAnchor" : false,
              "parentGUID" : "parent GUID, if set, set all parameters beginning 'parent'",
              "parentRelationshipTypeName" : "open metadata type name",
              "parentAtEnd1": true,
              "collectionProperties": {
                "class" : "CollectionProperties",
                "qualifiedName": "Must provide a unique name here",
                "name" : "Add display name here",
                "description" : "Add description of the collection here",
                "collectionType": "Add appropriate valid value for type",
                "collectionOrdering" : "OTHER",
                "orderPropertyName" : "Add property name if 'collectionOrdering' is OTHER"
              },
              "digitalProductProperties" : {
                "class" : "DigitalProductProperties",
                "productStatus" : "ACTIVE",
                "productName" : "Add name here",
                "productType" : "Add valid value here",
                "description" : "Add description here",
                "introductionDate" : "date",
                "maturity" : "Add valid value here",
                "serviceLife" : "Add the estimated lifetime of the product",
                "currentVersion": "V1.0",
                "nextVersion": "V1.1",
                "withdrawDate": "date",
                "additionalProperties": {
                  "property1Name" : "property1Value",
                  "property2Name" : "property2Value"
                }
              }
            }
    """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(self._async_create_digital_product(body, server_name))
        return resp

    #
    # Manage collections
    #
    async def _async_update_collection(self, collection_guid: str, qualified_name: str = None, display_name: str = None,
                                       description: str = None, collection_type: str = None,
                                       collection_ordering: str = None, order_property_name: str = None,
                                       replace_all_props: bool = False, server_name: str = None) -> None:
        """ Update the properties of a collection.  Async version.

            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            qualified_name: str, optional, defaults to None
                The qualified name of the collection to update.
            display_name: str, optional, defaults to None
               The display name of the element. Will also be used as the basis of the qualified_name.
            description: str, optional, defaults to None
               A description of the collection.
            collection_type: str, optional, defaults to None
               Add appropriate valid value for the collection type.
            collection_ordering: str, optional, defaults to None
               Specifies the sequencing to use in a collection. Examples include "NAME", "OWNER",
               "DATE_CREATED", "OTHER"
            order_property_name: str, optional, defaults to None
               Property to use for sequencing if collection_ordering is "OTHER"
            replace_all_props: bool, optional, defaults to False
                Whether to replace all properties in the collection.
            server_name: str, optional, defaults to None
               The name of the server to  configure. If not provided, the server name associated
               with the instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
             If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
             Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
             The principle specified by the user_id does not have authorization for the requested action
       """
        if server_name is None:
            server_name = self.server_name
        replace_all_props_s = str(replace_all_props).lower()
        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/{collection_guid}/update?"
               f"replaceAllProperties={replace_all_props_s}")

        body = {"class": "CollectionProperties", "qualifiedName": qualified_name, "name": display_name,
                "description": description, "collectionType": collection_type,
                "collectionOrdering": collection_ordering,
                "orderPropertyName": order_property_name}
        body_s = body_slimmer(body)
        await self._async_make_request("POST", url, body_s)
        return

    def update_collection(self, collection_guid, qualified_name: str = None, display_name: str = None,
                          description: str = None, collection_type: str = None, collection_ordering: str = None,
                          order_property_name: str = None, replace_all_props: bool = False,
                          server_name: str = None) -> None:
        """ Update the properties of a collection.

            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            qualified_name: str
                The qualified name of the collection to update.
            display_name: str
               The display name of the element. Will also be used as the basis of the qualified_name.
            description: str
               A description of the collection.
            collection_type: str
               Add appropriate valid value for the collection type.
            collection_ordering: str, optional, defaults to "OTHER"
               Specifies the sequencing to use in a collection. Examples include "NAME", "OWNER",
               "DATE_CREATED", "OTHER"
            order_property_name: str, optional, defaults to "Something"
               Property to use for sequencing if collection_ordering is "OTHER"
            replace_all_props: bool, optional, defaults to False
                Whether to replace all properties in the collection.
            server_name: str, optional, defaults to None
               The name of the server to  configure. If not provided, the server name associated
               with the instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
             If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
             Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
             The principle specified by the user_id does not have authorization for the requested action
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_collection(collection_guid, qualified_name, display_name, description, collection_type,
                                          collection_ordering, order_property_name, replace_all_props, server_name))
        return

    async def _async_update_digital_product(self, collection_guid: str, body: dict, replace_all_props: bool = False,
                                            server_name: str = None):
        """ Update the properties of the DigitalProduct classification attached to a collection. Async version.

            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            body: dict
                A dict representing the details of the collection to create.
            replace_all_props: bool, optional, defaults to False
                Whether to replace all properties in the collection.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated with the
                instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            JSON Structure looks like:
            {
              "class" : "DigitalProductProperties",
              "productStatus" : "ACTIVE",
              "productName" : "Add name here",
              "productType" : "Add valid value here",
              "description" : "Add description here",
              "introductionDate" : "date",
              "maturity" : "Add valid value here",
              "serviceLife" : "Add the estimated lifetime of the product",
              "currentVersion": "V1.0",
              "nextVersion": "V1.1",
              "withdrawDate": "date",
              "additionalProperties": {
                "property1Name" : "property1Value",
                "property2Name" : "property2Value"
              }
            }
        """
        if server_name is None:
            server_name = self.server_name

        replace_all_props_s = str(replace_all_props).lower()
        url = (f"{self.platform_url}/servers/{server_name}/api/open-metadata/collection-manager/digital-products/"
               f"{collection_guid}/update?replaceAllProperties={replace_all_props_s}")

        await self._async_make_request("POST", url, body)
        return

    def update_digital_product(self, collection_guid: str, body: dict, replace_all_props: bool = False,
                               server_name: str = None):
        """ Update the properties of the DigitalProduct classification attached to a collection.

            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            body: dict
                A dict representing the details of the collection to create.
            replace_all_props: bool, optional, defaults to False
                Whether to replace all properties in the collection.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated with the
                instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            JSON Structure looks like:
            {
              "class" : "DigitalProductProperties",
              "productStatus" : "ACTIVE",
              "productName" : "Add name here",
              "productType" : "Add valid value here",
              "description" : "Add description here",
              "introductionDate" : "date",
              "maturity" : "Add valid value here",
              "serviceLife" : "Add the estimated lifetime of the product",
              "currentVersion": "V1.0",
              "nextVersion": "V1.1",
              "withdrawDate": "date",
              "additionalProperties": {
                "property1Name" : "property1Value",
                "property2Name" : "property2Value"
              }
            }
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_digital_product(collection_guid, body, replace_all_props, server_name))
        return

    async def _async_attach_collection(self, collection_guid: str, element_guid: str, resource_use: str,
                                       resource_use_description: str = None, resource_use_props: dict = None,
                                       watch_resources: bool = False, make_anchor: bool = False,
                                       server_name: str = None) -> None:
        """ Connect an existing collection to an element using the ResourceList relationship (0019). Async version.
            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            element_guid: str
                The guid of the element to attach.
            resource_use: str,
                How the resource is being used.
            resource_use_description: str
                Describe how the resource is being used.
            resource_use_props: dict, optional, defaults to None
                The properties of the resource to be used.
            watch_resources, bool, optional, defaults to False
                Whether to watch for the resources to be updated.
            make_anchor, bool, optional, defaults to False
                Whether to make the this an anchor.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated with the
                instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        watch_resources_s = str(watch_resources).lower()
        make_anchor_s = str(make_anchor).lower()

        url = (f"{self.platform_url}/servers/{server_name}/api/open-metadata/collection-manager/metadata-elements/"
               f"{element_guid}/collections/{collection_guid}/attach?makeAnchor={make_anchor_s}")

        body = {"class": "ResourceListProperties", "resourceUse": resource_use,
                "resourceUseDescription": resource_use_description, "watchResource": watch_resources_s,
                "resourceUseProperties": resource_use_props}
        await self._async_make_request("POST", url, body)
        return

    def attach_collection(self, collection_guid: str, element_guid: str, resource_use: str,
                          resource_use_description: str, resource_use_props: dict = None, watch_resources: bool = False,
                          make_anchor: bool = False, server_name: str = None) -> None:
        """ Connect an existing collection to an element using the ResourceList relationship (0019).
            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            element_guid: str
                The guid of the element to attach.
            resource_use: str,
                How the resource is being used.
            resource_use_description: str
                Describe how the resource is being used.
            resource_use_props: dict, optional, defaults to None
                The properties of the resource to be used.
            watch_resources: bool, optional, defaults to False
                Whether to watch for the resources to be updated.
            make_anchor: bool, optional, defaults to False
                Whether to make the this an anchor.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated
                with the instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_attach_collection(collection_guid, element_guid, resource_use, resource_use_description,
                                          resource_use_props, watch_resources, make_anchor, server_name))
        return

    async def _async_detach_collection(self, collection_guid: str, element_guid: str, server_name: str = None) -> None:
        """ Detach an existing collection from an element.  If the collection is anchored to the element, it is deleted.
            Async version.

            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            element_guid: str
                The guid of the element to attach.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated
                with the instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = (f"{self.platform_url}/servers/{server_name}/api/open-metadata/collection-manager/metadata-elements/"
               f"{element_guid}/collections/{collection_guid}/detach")

        body = {"class": "NullRequestBody"}

        await self._async_make_request("POST", url, body)
        return

    def detach_collection(self, collection_guid: str, element_guid: str, server_name: str = None) -> None:
        """ Connect an existing collection to an element using the ResourceList relationship (0019).
            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            element_guid: str
                The guid of the element to attach.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated
                with the instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_detach_collection(collection_guid, element_guid, server_name))
        return

    async def _async_delete_collection(self, collection_guid: str, server_name: str = None) -> None:
        """ Delete a collection.  It is detected from all parent elements.  If members are anchored to the collection
            then they are also deleted. Async version


            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated
                with the instance is used.

            Returns
            -------
            Nothing

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/{collection_guid}/delete"
        body = {"class": "NullRequestBody"}

        await self._async_make_request("POST", url, body)
        return

    def delete_collection(self, collection_guid: str, server_name: str = None) -> None:
        """ Delete a collection.  It is detected from all parent elements.  If members are anchored to the collection
            then they are also deleted.

            Parameters
            ----------
            collection_guid: str
                The guid of the collection to update.
            server_name: str, optional, defaults to None
                The name of the server to  configure. If not provided, the server name associated
                with the instance is used.

            Returns
            -------
            Nothing

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_delete_collection(collection_guid, server_name))
        return

    async def _async_get_collection_members(self, collection_guid: str, effective_time: str = None,
                                            server_name: str = None, start_from: int = 0,
                                            page_size: int = None) -> list | str:
        """ Return a list of elements that are a member of a collection. Async version.

        Parameters
        ----------
        collection_guid: str,
            identity of the collection to return members for.
        effective_time: str, [default=None], optional
            Effective time of the query. If not specified will default to any time.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List | str

        A list of collection members in the collection.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        if page_size is None:
            page_size = self.page_size

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/{collection_guid}/"
               f"members?startFrom={start_from}&pageSize={page_size}")

        resp = await self._async_make_request("GET", url)
        return resp.json().get("elements", "No elements found")

    def get_collection_members(self, collection_guid: str, effective_time: str = None, server_name: str = None,
                               start_from: int = 0, page_size: int = None) -> list | str:
        """ Return a list of elements that are a member of a collection.

            Parameters
            ----------
            collection_guid: str,
                identity of the collection to return members for.
            effective_time: str, [default=None], optional
                Effective time of the query. If not specified will default to any time.
            server_name : str, optional
                The name of the server to  configure.
                If not provided, the server name associated with the instance is used.
            start_from: int, [default=0], optional
                        When multiple pages of results are available, the page number to start from.
            page_size: int, [default=None]
                The number of items to return in a single page. If not specified, the default will be taken from
                the class instance.
            Returns
            -------
            List | str

            A list of collection members in the collection.

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_collection_members(collection_guid, effective_time, server_name, start_from, page_size))

        return resp

    async def _async_add_to_collection(self, collection_guid: str, element_guid: str, body: dict = None,
                                       server_name: str = None) -> None:
        """ Add an element to a collection.  The request body is optional. Async version.

            Parameters
            ----------
            collection_guid: str
                identity of the collection to return members for.
            element_guid: str
                Effective time of the query. If not specified will default to any time.
            body: dict, optional, defaults to None
                The body of the request to add to the collection. See notes.
            server_name : str, optional
                The name of the server to use.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            None

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            Example body:
            {
              "class" : "CollectionMembershipProperties",
              "membershipRationale": "xxx",
              "createdBy": "user id here",
              "expression": "expression that described why the element is a part of this collection",
              "confidence": 100,
              "status": "PROPOSED",
              "userDefinedStatus": "Add valid value here",
              "steward": "identifier of steward that validated this member",
              "stewardTypeName": "type name of element identifying the steward",
              "stewardPropertyName": "property name if the steward's identifier",
              "source": "source of the member",
              "notes": "Add notes here"
            }

        """
        if server_name is None:
            server_name = self.server_name

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/{collection_guid}/members/"
               f"{element_guid}/attach")
        body_s = body_slimmer(body)
        await self._async_make_request("POST", url, body_s)
        return

    def add_to_collection(self, collection_guid: str, element_guid: str, body: dict = None,
                          server_name: str = None) -> None:
        """ Add an element to a collection.  The request body is optional.

            Parameters
            ----------
            collection_guid: str
                identity of the collection to return members for.
            element_guid: str
                Effective time of the query. If not specified will default to any time.
            body: dict, optional, defaults to None
                The body of the request to add to the collection. See notes.
            server_name : str, optional
                The name of the server to use.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            None

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            Example body:
            {
              "class" : "CollectionMembershipProperties",
              "membershipRationale": "xxx",
              "createdBy": "user id here",
              "expression": "expression that described why the element is a part of this collection",
              "confidence": 100,
              "status": "PROPOSED",
              "userDefinedStatus": "Add valid value here",
              "steward": "identifier of steward that validated this member",
              "stewardTypeName": "type name of element identifying the steward",
              "stewardPropertyName": "property name if the steward's identifier",
              "source": "source of the member",
              "notes": "Add notes here"
            }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_add_to_collection(collection_guid, element_guid, body, server_name))
        return

    async def _async_update_collection_membership(self, collection_guid: str, element_guid: str, body: dict = None,
                                                  replace_all_props: bool = False, server_name: str = None) -> None:
        """ Update an element's membership to a collection. Async version.

            Parameters
            ----------
            collection_guid: str
                identity of the collection to return members for.
            element_guid: str
                Effective time of the query. If not specified will default to any time.
            body: dict, optional, defaults to None
                The body of the request to add to the collection. See notes.
            replace_all_props: bool, optional, defaults to False
                Replace all properties or just update ones specified in body.
            server_name : str, optional
                The name of the server to use.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            None

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            Example body:
            {
              "class" : "CollectionMembershipProperties",
              "membershipRationale": "xxx",
              "createdBy": "user id here",
              "expression": "expression that described why the element is a part of this collection",
              "confidence": 100,
              "status": "PROPOSED",
              "userDefinedStatus": "Add valid value here",
              "steward": "identifier of steward that validated this member",
              "stewardTypeName": "type name of element identifying the steward",
              "stewardPropertyName": "property name if the steward's identifier",
              "source": "source of the member",
              "notes": "Add notes here"
            }

        """
        if server_name is None:
            server_name = self.server_name
        replace_all_props_s = str(replace_all_props).lower()
        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/{collection_guid}/members/"
               f"{element_guid}/update?replaceAllProperties={replace_all_props_s}")
        body_s = body_slimmer(body)
        await self._async_make_request("POST", url, body_s)
        return

    def update_collection_membership(self, collection_guid: str, element_guid: str, body: dict = None,
                                     replace_all_props: bool = False, server_name: str = None) -> None:
        """ Update an element's membership to a collection.

            Parameters
            ----------
            collection_guid: str
                identity of the collection to update members for.
            element_guid: str
                Effective time of the query. If not specified will default to any time.
            body: dict, optional, defaults to None
                The body of the request to add to the collection. See notes.
            replace_all_props: bool, optional, defaults to False
                Replace all properties or just update ones specified in body.
            server_name : str, optional
                The name of the server to use.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            None

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----
            Example body:
            {
              "class" : "CollectionMembershipProperties",
              "membershipRationale": "xxx",
              "createdBy": "user id here",
              "expression": "expression that described why the element is a part of this collection",
              "confidence": 100,
              "status": "PROPOSED",
              "userDefinedStatus": "Add valid value here",
              "steward": "identifier of steward that validated this member",
              "stewardTypeName": "type name of element identifying the steward",
              "stewardPropertyName": "property name if the steward's identifier",
              "source": "source of the member",
              "notes": "Add notes here"
            }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_collection_membership(collection_guid, element_guid, body, replace_all_props,
                                                     server_name))
        return

    async def _async_remove_from_collection(self, collection_guid: str, element_guid: str,
                                            server_name: str = None) -> None:
        """ Remove an element from a collection. Async version.

            Parameters
            ----------
            collection_guid: str
                identity of the collection to return members for.
            element_guid: str
                Effective time of the query. If not specified will default to any time.
            server_name : str, optional
                The name of the server to use.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            None

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/{collection_guid}/members/"
               f"{element_guid}/detach")
        body = {"class": "NullRequestBody"}
        await self._async_make_request("POST", url, body)
        return

    def remove_from_collection(self, collection_guid: str, element_guid: str, server_name: str = None) -> None:
        """ Remove an element from a collection.

            Parameters
            ----------
            collection_guid: str
                identity of the collection to return members for.
            element_guid: str
                Effective time of the query. If not specified will default to any time.
            server_name : str, optional
                The name of the server to use.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            None

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_remove_from_collection(collection_guid, element_guid, server_name))
        return

    async def _async_get_member_list(self, root_collection_name: str, server_name: str = None) -> list | bool:
        """ Get the member list for the collection - async version.
        Parameters
        ----------
        root_collection_name : str
            The name of the root collection.

        server_name : str, optional
            The name of the server. If not provided, the default server name will be used.

        Returns
        -------
        list | bool
            The list of member information if successful, otherwise False.

        Raises
        ------
        InvalidParameterException
            If the root_collection_name does not have exactly one root collection.

        """
        if server_name is None:
            server_name = self.server_name
        # first find the guid for the collection we are using as root
        root_guids = await self._async_get_collections_by_name(root_collection_name)
        if type(root_guids) is str:
            return False
        if len(root_guids) != 1:
            raise InvalidParameterException(
                "root_collection_name must have exactly one root collection for this method")
        root = root_guids[0]['elementHeader']['guid']

        # now find the members of the collection
        member_list = []
        members = await self._async_get_collection_members(root)
        if type(members) is str:
            return False
        # finally, construct a list of  member information
        for member_rel in members:
            member_guid = member_rel['elementHeader']['guid']
            member_resp = await self._async_get_collection(member_guid)
            member = member_resp['element']
            # print(json.dumps(member, indent = 4))
            member_instance = {"name": member['properties']['name'],
                               "qualifiedName": member['properties']['qualifiedName'],
                               "guid": member['elementHeader']['guid'],
                               "description": member['properties']['description'],
                               "collectionType": member['properties']['collectionType'], }
            member_list.append(member_instance)

        return member_list

    def get_member_list(self, root_collection_name: str, server_name: str = None) -> list | bool:
        """ Get the member list for the collection.
        Parameters
        ----------
        root_collection_name : str
            The name of the root collection.

        server_name : str, optional
            The name of the server. If not provided, the default server name will be used.

        Returns
        -------
        list | bool
            The list of member information if successful, otherwise False.

        Raises
        ------
        InvalidParameterException
            If the root_collection_name does not have exactly one root collection.

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(self._async_get_member_list(root_collection_name, server_name))
        return resp



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A command line interface for Egeria operations.

This is an emerging capability based on the **click** package. Feedback welcome!

"""
import click
from trogon import tui

# from pyegeria import ServerOps
from examples.widgets.cli.ops_config import Config
from examples.widgets.ops.engine_actions import start_server as start_engine_host, \
    stop_server as stop_engine_host
from examples.widgets.ops.integration_daemon_actions import (add_catalog_target, remove_catalog_target,
                                                             update_catalog_target, stop_server, start_server)
from examples.widgets.ops.list_catalog_targets import display_catalog_targets
from examples.widgets.ops.load_archive import load_archive
from examples.widgets.ops.monitor_engine_activity import display_engine_activity
from examples.widgets.ops.monitor_gov_eng_status import display_gov_eng_status
from examples.widgets.ops.monitor_integ_daemon_status import display_integration_daemon_status
from examples.widgets.ops.monitor_platform_status import display_status as p_display_status
from examples.widgets.ops.monitor_server_list import display_status as display_list
from examples.widgets.ops.monitor_server_status import display_status as s_display_status
from examples.widgets.ops.refresh_integration_daemon import refresh_connector
from examples.widgets.ops.restart_integration_daemon import restart_connector


# class Config(object):
#     def __init__(self, server: str = None, url: str = None, userid:str = None, password:str = None,
#                  timeout:int = 30, paging: bool = False):
#         self.server = server
#         self.url = url
#         self.userid = userid
#         self.password = password
#         self.timeout = timeout
#         self.paging = paging
#
#
# pass_config = click.make_pass_decorator(Config)

@tui()
# @tui('menu', 'menu', 'A textual command line interface')
@click.version_option("0.0.1", prog_name="egeria_ops")
@click.group()
@click.option('--server', default='active-metadata-store', envvar='EGERIA_METADATA_STORE',
              help='Egeria metadata store to work with')
@click.option('--url', default='https://localhost:9443', envvar='EGERIA_PLATFORM_URL',
              help='URL of Egeria metadata store platform to connect to')
@click.option('--integration-daemon', default='integration-daemon', envvar='EGERIA_INTEGRATION_DAEMON',
              help='Egeria integration daemon to work with')
@click.option('--integration_daemon_url', default='https://localhost:9443', envvar='EGERIA_INTEGRATION_DAEMON_URL',
              help='URL of Egeria integration daemon platform to connect to')
@click.option('--view_server', default='view-server', envvar='EGERIA_VIEW_SERVER',
              help='Egeria view server to work with')
@click.option('--view_server_url', default='https://localhost:9443', envvar='EGERIA_VIEW_SERVER_URL',
              help='URL of Egeria view server platform to connect to')
@click.option('--engine_host', default='engine-host', envvar='EGERIA_ENGINE_HOST',
              help='Egeria engine host to work with')
@click.option('--engine_host_url', default='https://localhost:9443', envvar='EGERIA_ENGINE_HOST_URL',
              help='URL of Egeria engine host platform to connect to')
@click.option('--admin_user', default='garygeeke', envvar='EGERIA_ADMIN_USER', help='Egeria admin user')
@click.option('--admin_user_password', default='secret', envvar='EGERIA_ADMIN_PASSWORD',
              help='Egeria admin password')
@click.option('--userid', default='erinoverview', envvar='EGERIA_USER', help='Egeria user')
@click.option('--password', default='secret', envvar='EGERIA_PASSWORD',
              help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.option('--verbose', is_flag=True, default=False, help='Enable verbose mode')
@click.option('--paging', is_flag=True, default=False, help='Enable paging snapshots vs live updates')
@click.option('--jupyter', is_flag=True, default=False, envvar='EGERIA_JUPYTER',
              help='Enable for rendering in a Jupyter terminal')
@click.option('--width', default=200, envvar='EGERIA_WIDTH', help='Screen width, in characters, to use')
@click.pass_context
def cli(ctx, server, url, view_server, view_server_url, integration_daemon, integration_daemon_url,
        engine_host, engine_host_url, admin_user, admin_user_password, userid, password, timeout, paging,
        verbose, jupyter, width):
    """An Egeria Command Line interface for Operations """
    ctx.obj = Config(server, url, view_server, view_server_url, integration_daemon,
                     integration_daemon_url, engine_host, engine_host_url,
                     admin_user, admin_user_password, userid, password,
                     timeout, paging, verbose, jupyter, width)
    ctx.max_content_width = 200
    ctx.ensure_object(Config)
    if verbose:
        click.echo(f"we are in verbose mode - server is {server}")


@cli.group("show")
@click.pass_context
def show(ctx):
    """Display an Egeria Object"""
    pass


@show.group('platforms')
@click.pass_context
def show_platform(ctx):
    """Group of commands to show information about Egeria platforms"""
    pass


@show_platform.command('status')
@click.pass_context
def show_platform_status(ctx):
    """Display a live status view of known platforms"""
    c = ctx.obj
    p_display_status(c.view_server, c.view_server_url,
                     c.admin_user, c.admin_user_password)


@show.group("servers")
@click.pass_context
def show_server(ctx):
    """Group of commands to show information about Egeria servers"""
    pass


@show_server.command('status')
@click.option('--full', is_flag=True, default=False, help='If True, full server descriptions will be shown')
@click.pass_context
def show_server_status(ctx, full):
    """Display a live status view of Egeria servers for the specified Egeria platform"""
    c = ctx.obj
    if full:
        display_list(c.metadata_store, c.metadata_store_url, c.admin_user, c.admin_user_password, c.jupyter, c.width)
    else:
        s_display_status(c.metadata_store, c.metadata_store_url, c.admin_user, c.admin_user_password, c.jupyter,
                         c.width)


@show.group("engines")
@click.pass_context
def engine_host(ctx):
    """Group of commands to show information about Egeria engines"""
    pass


@engine_host.command("status")
@click.option('--list', is_flag=True, default=False, help='If True, a paged list will be shown')
@click.pass_context
def gov_eng_status(ctx, list):
    """Display engine-host status information"""
    c = ctx.obj
    display_gov_eng_status(c.engine_host, c.engine_host_url,
                           c.userid, c.password,
                           list, c.jupyter, c.width)


@engine_host.command('activity')
@click.option('--list', is_flag=True, default=False, help='If True, a paged list will be shown')
@click.pass_context
def eng_activity_status(ctx, list):
    """Show Governance Activity in engine-host"""
    c = ctx.obj
    display_engine_activity(c.view_server, c.view_server_url,
                            c.admin_user, c.admin_user_password,
                            list, c.jupyter, c.width)


@show.group('integrations')
@click.pass_context
def integrations(ctx):
    """Group of commands to show information about Egeria integrations"""
    pass


@integrations.command("status")
@click.option('--list', is_flag=True, default=False, help='If True, a paged list will be shown')
@click.pass_context
def integrations_status(ctx, list):
    """Display integration-daemon status information"""
    c = ctx.obj
    display_integration_daemon_status(c.integration_daemon, c.integration_daemon_url,
                                      c.view_server, c.view_server_url,
                                      c.userid, c.password, list, c.jupyter, c.width)


@integrations.command("targets")
@click.pass_context
@click.argument('connector', nargs=1)
def integrations_status(ctx, connector):
    """Display Catalog Targets for a connector"""
    c = ctx.obj
    display_catalog_targets(connector, c.view_server, c.view_server_url,
                            c.userid, c.password, c.jupyter, c.width)


#
#  Tell
#

@cli.group('tell')
@click.pass_context
def tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


@tell.group('integration-daemon')
@click.pass_context
def integration_daemon(ctx):
    """Group of commands to an integration-daemon"""
    pass


@integration_daemon.command('refresh')
@click.pass_context
@click.option('--connector', default='all', help="Name of connector to refresh or 'all' to refresh all")
def refresh_connectors(ctx, connector):
    """Refresh the specified integration connector or ALL connectors if not specified"""
    c = ctx.obj
    refresh_connector(connector, c.integration_daemon, c.integration_daemon_url,
                      c.userid, c.password)


@integration_daemon.command('restart')
@click.pass_context
@click.option('--connector', default='all', help="Name of connector to restart or 'all' to restart all")
def restart_connectors(ctx, connector):
    """Restart the specified integration connector or ALL connectors if not specified"""
    c = ctx.obj
    restart_connector(connector, c.integration_daemon, c.integration_daemon_url,
                      c.userid, c.password)


integration_daemon.add_command(add_catalog_target)
integration_daemon.add_command(remove_catalog_target)
integration_daemon.add_command(update_catalog_target)
integration_daemon.add_command(stop_server)
integration_daemon.add_command(start_server)


@tell.group('engine-host')
@click.pass_context
def engine_host(ctx):
    """Group of commands to an engine-host"""
    pass


engine_host.add_command(start_engine_host)
engine_host.add_command(stop_engine_host)

tell.add_command(load_archive)

if __name__ == '__main__':
    cli()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


List catalog targets
"""
import argparse
import os
import time

from rich import box
from rich.console import Console
from rich.markdown import Markdown
from rich.prompt import Prompt
from rich.table import Table

from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
    AutomatedCuration,
    INTEGRATION_GUIDS
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')
EGERIA_JUPYTER = bool(os.environ.get('EGERIA_JUPYTER', 'False'))
EGERIA_WIDTH = int(os.environ.get('EGERIA_WIDTH', '200'))



def display_catalog_targets(connector: str, server: str, url: str, username: str, user_password: str,
                            jupyter: bool = EGERIA_JUPYTER, width: int = EGERIA_WIDTH):
    console = Console(force_terminal=not jupyter, width=width, soft_wrap=True)
    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"All Catalog Targets for Integration Connector {connector} @ {time.asctime()}",
            style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"Catalog Targets for '{server}' @ Platform - {url}",
            expand=True
        )
        table.add_column("Name of Target")
        table.add_column("Catalog Target")

        # table.add_column("Relationship GUID", no_wrap=True)
        table.add_column('Configuration Properties')
        table.add_column('Template Properties')
        table.add_column("Operational Instructions", max_width=20)
        # table.add_column("Delete Method")

        if type(cat_targets) is list:
            for target in cat_targets:
                target_name = target.get('catalogTargetName', '---')
                target_source = target.get('metadataSourceQualifiedName', '---')
                target_rel = target.get('relationshipGUID', '---')
                target_sync = target.get('permittedSynchronization')
                target_delete = target.get('deleteMethod', '---')
                op_instruct = f"* {target_sync}\n* {target_delete}"
                op_instruct_out = Markdown(op_instruct)
                # target_guid = target['catalogTargetElement']['guid']
                connector_unique = target['catalogTargetElement']['uniqueName']

                cat_target_out = Markdown(f"* Target Name: {target_name}\n* Target Source: {target_source}\n"
                                          f"* Relationship Guid: {target_rel}")

                config_props = target.get('configurationProperties', '---')
                if type(config_props) is dict:
                    config_props_md = ''
                    for prop in config_props:
                        config_props_md += f"* {prop}: {config_props[prop]}\n"
                    config_props_out = Markdown(config_props_md)
                else:
                    config_props_out = '---'

                template_props = target.get('templateProperties', '---')
                if type(template_props) is dict:
                    template_props_md = ''
                    for prop in template_props:
                        template_props_md += f"* {prop}: {template_props[prop]}\n"
                    template_props_out = Markdown(template_props_md)
                else:
                    template_props_out = '---'

                table.add_row(
                    connector_unique, cat_target_out,
                    config_props_out, template_props_out, op_instruct_out)

        return table

    try:
        a_client = AutomatedCuration(server, url, username)
        token = a_client.create_egeria_bearer_token(username, user_password)
        if connector not in INTEGRATION_GUIDS.keys():
            raise Exception

        connector_guid = INTEGRATION_GUIDS[connector]
        cat_targets = a_client.get_catalog_targets(connector_guid)
        console = Console(force_terminal=not jupyter, width=width, soft_wrap=True)
        with console.pager(styles=True):
            console.print(generate_table(), soft_wrap=True)

    except (InvalidParameterException, PropertyServerException, UserNotAuthorizedException) as e:
        print_exception_response(e)
    except Exception as e:
        print(f"\n\n===> Perhaps integration connector {connector} is not known?\n\n")
        console.print_exception()
    finally:
        a_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    parser.add_argument("--jupyter", help="Enable for Jupyter terminal compatibility")
    parser.add_argument("--width", help="Screen width, in characters, to use")

    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_VIEW_SERVER_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    jupyter = args.jupyter if args.jupyter is not None else EGERIA_JUPYTER
    width = args.width if args.width is not None else EGERIA_WIDTH

    try:
        connector = Prompt.ask("Enter the Integration Connector to list catalog targets for")
        display_catalog_targets(connector, server, url, userid, user_pass, jupyter, width=width)

    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This script restarts an integration daemon.

"""

import os
from rich import print, print_json
from rich.console import Console

import click
from pyegeria import ServerOps, AutomatedCuration, INTEGRATION_GUIDS, Platform
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)


@click.command('add-todo')
@click.argument('integration-connector')
@click.argument('metadata-element-guid')
@click.argument('catalog-target-name')
@click.pass_context
def add_catalog_target(ctx, integration_connector: str, metadata_element_guid:str, catalog_target_name:str)-> str:
    """Add catalog targets to the specified integration connector"""
    try:
        if integration_connector not in INTEGRATION_GUIDS.keys():
            click.echo('Integration connector is not known')

        c = ctx.obj
        a_client = AutomatedCuration(c.view_server, c.view_server_url, c.userid, c.password)
        token = a_client.create_egeria_bearer_token()

        guid = a_client.add_catalog_target(INTEGRATION_GUIDS[integration_connector], metadata_element_guid,
                                           catalog_target_name)

        click.echo(f"Added catalog target to {integration_connector} with a return guid of {guid}")


    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)



@click.command('remove-target')
@click.argument('relationship-guid')
@click.pass_context
def remove_catalog_target(ctx, relationship_guid: str):
    """Remove the catalog target specified by the relationship guidr"""
    try:
        c = ctx.obj
        a_client = AutomatedCuration(c.view_server, c.view_server_url, c.userid, c.password)
        token = a_client.create_egeria_bearer_token()

        a_client.remove_catalog_target(relationship_guid)

        click.echo(f"Removed catalog target with relationship guid of {relationship_guid}")


    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)


@click.command('update-target')
@click.argument('relationship-guid')
@click.argument('catalog-target-name')
@click.pass_context
def update_catalog_target(ctx, relationship_guid: str, catalog_target_name:str):
    """Update the catalog target specified by the relationship guid """
    try:
        c = ctx.obj
        a_client = AutomatedCuration(c.view_server, c.view_server_url, c.userid, c.password)
        token = a_client.create_egeria_bearer_token()

        guid = a_client.update_catalog_target(relationship_guid, catalog_target_name)

        click.echo(f"Update catalog target with relationship guid of  {relationship_guid} to a catalog target name of "
                   f"{catalog_target_name} with a return guid of {guid}")


    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)


@click.command('stop')
@click.pass_context
def stop_server(ctx):
    """Stop an integration daemon"""
    try:
        c = ctx.obj
        p_client = Platform(c.integration_daemon, c.integration_daemon_url, c.userid, c.password)

        p_client.shutdown_server()

        click.echo(f"Stopped server {c.integration_daemon}")
    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)


@click.command('start')
@click.pass_context
def start_server(ctx):
    """Start or restart an integration daemon from its known configuration """
    try:
        c = ctx.obj
        p_client = Platform(c.integration_daemon, c.integration_daemon_url, c.userid, c.password)

        p_client.activate_server_stored_config()

        click.echo(f"Started server {c.integration_daemon}")

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)




#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A command line interface for Egeria Users - all commands

This is an emerging capability based on the **click** package. Feedback welcome!

"""
import click
from trogon import tui

from examples.widgets.cat.get_project_structure import project_structure_viewer
from examples.widgets.cat.list_cert_types import display_certifications
from examples.widgets.cat.get_asset_graph import asset_viewer
from examples.widgets.cat.get_collection import collection_viewer
from examples.widgets.cat.get_tech_type_elements import tech_viewer
from examples.widgets.cat.get_tech_type_template import template_viewer
from examples.widgets.cat.list_assets import display_assets
from examples.widgets.cat.list_glossary import display_glossary_terms
from examples.widgets.cat.list_projects import display_project_list
from examples.widgets.cat.list_tech_types import display_tech_types
from examples.widgets.cat.list_todos import display_to_dos
from examples.widgets.cli.ops_config import Config

from examples.widgets.ops.engine_actions import start_server as start_engine_host, \
    stop_server as stop_engine_host
from examples.widgets.ops.integration_daemon_actions import (add_catalog_target, remove_catalog_target,
                                                             update_catalog_target, stop_server, start_server)
from examples.widgets.ops.list_catalog_targets import display_catalog_targets
from examples.widgets.ops.load_archive import load_archive
from examples.widgets.ops.monitor_engine_activity import display_engine_activity
from examples.widgets.ops.monitor_gov_eng_status import display_gov_eng_status
from examples.widgets.ops.monitor_integ_daemon_status import display_integration_daemon_status
from examples.widgets.ops.monitor_platform_status import display_status as p_display_status
from examples.widgets.ops.monitor_server_list import display_status as display_list
from examples.widgets.ops.monitor_server_status import display_status as s_display_status
from examples.widgets.ops.refresh_integration_daemon import refresh_connector
from examples.widgets.ops.restart_integration_daemon import restart_connector
from examples.widgets.my.list_my_profile import display_my_profile
from examples.widgets.my.list_my_roles import display_my_roles
from examples.widgets.my.monitor_my_todos import display_my_todos
from examples.widgets.my.monitor_open_todos import display_todos
from examples.widgets.tech.get_guid_info import display_guid
from examples.widgets.tech.get_tech_details import tech_details_viewer
from examples.widgets.tech.list_asset_types import display_asset_types
from examples.widgets.tech.list_registered_services import display_registered_svcs
from examples.widgets.tech.list_relationship_types import display_relationship_types
from examples.widgets.tech.list_tech_templates import display_templates_spec
from examples.widgets.tech.list_valid_metadata_values import display_metadata_values


@tui()
# @tui('menu', 'menu', 'A textual command line interface')
@click.version_option("0.0.1", prog_name="egeria_ops")
@click.group()
@click.option('--server', default='active-metadata-store', envvar='EGERIA_METADATA_STORE',
              help='Egeria metadata store to work with')
@click.option('--url', default='https://localhost:9443', envvar='EGERIA_PLATFORM_URL',
              help='URL of Egeria metadata store platform to connect to')
@click.option('--integration-daemon', default='integration-daemon', envvar='EGERIA_INTEGRATION_DAEMON',
              help='Egeria integration daemon to work with')
@click.option('--integration_daemon_url', default='https://localhost:9443', envvar='EGERIA_INTEGRATION_DAEMON_URL',
              help='URL of Egeria integration daemon platform to connect to')
@click.option('--view_server', default='view-server', envvar='EGERIA_VIEW_SERVER',
              help='Egeria view server to work with')
@click.option('--view_server_url', default='https://localhost:9443', envvar='EGERIA_VIEW_SERVER_URL',
              help='URL of Egeria view server platform to connect to')
@click.option('--engine_host', default='engine-host', envvar='EGERIA_ENGINE_HOST',
              help='Egeria engine host to work with')
@click.option('--engine_host_url', default='https://localhost:9443', envvar='EGERIA_ENGINE_HOST_URL',
              help='URL of Egeria engine host platform to connect to')
@click.option('--admin_user', default='garygeeke', envvar='EGERIA_ADMIN_USER', help='Egeria admin user')
@click.option('--admin_user_password', default='secret', envvar='EGERIA_ADMIN_PASSWORD',
              help='Egeria admin password')
@click.option('--userid', default='erinoverview', envvar='EGERIA_USER', help='Egeria user')
@click.option('--password', default='secret', envvar='EGERIA_PASSWORD',
              help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.option('--verbose', is_flag=True, default=False, help='Enable verbose mode')
@click.option('--paging', is_flag=True, default=False, help='Enable paging snapshots vs live updates')
@click.option('--jupyter', is_flag=True, default=False, envvar='EGERIA_JUPYTER',
              help='Enable for rendering in a Jupyter terminal')
@click.option('--width', default=200, envvar='EGERIA_WIDTH', help='Screen width, in characters, to use')
@click.pass_context
def cli(ctx, server, url, view_server, view_server_url, integration_daemon, integration_daemon_url,
        engine_host, engine_host_url, admin_user, admin_user_password, userid, password, timeout, paging,
        verbose, jupyter, width):
    """An Egeria Command Line interface for Operations """
    ctx.obj = Config(server, url, view_server, view_server_url, integration_daemon,
                     integration_daemon_url, engine_host, engine_host_url,
                     admin_user, admin_user_password, userid, password,
                     timeout, paging, verbose, jupyter, width)
    ctx.max_content_width = 200
    ctx.ensure_object(Config)
    if verbose:
        click.echo(f"we are in verbose mode - server is {server}")


#
#  my: Show
#
@cli.group('my')
@click.pass_context
def my(ctx):
    "Work with my information"
    pass


@my.group("show")
@click.pass_context
def show(ctx):
    """Display an Egeria Object"""
    pass


@show.command('my-profile')
@click.pass_context
def show_my_profile(ctx):
    """Display my profiles

    Usage: show my-profile

    """
    c = ctx.obj
    display_my_profile(c.view_server, c.view_server_url,
                       c.userid, c.password, c.jupyter, c.width)


@show.command('my-roles')
@click.pass_context
def show_my_roles(ctx):
    """Display my profiles

    Usage: show my-profile

    """
    c = ctx.obj
    display_my_roles(c.view_server, c.view_server_url,
                     c.userid, c.password, c.jupyter, c.width)


@show.command('my-to-dos')
@click.pass_context
def show_my_todos(ctx):
    """Show my To-Dos

    Usage: show my-to-dos

    """
    c = ctx.obj
    display_my_todos(c.view_server, c.view_server_url,
                     c.userid, c.password, c.jupyter, c.width)


@show.command('open-to-dos')
@click.pass_context
def show_open_todos(ctx):
    """Display a live status view of Egeria servers for the specified Egeria platform

    Usage: show tech-details <tech-name>

           tech-name is a valid technology name (see 'show tech-types')
    """
    c = ctx.obj
    display_todos(c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width)


#
#  my: Tell
#

@my.group('tell')
@click.pass_context
def tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


#
#  tech User: Show
#
@cli.group("tech")
@click.pass_context
def tech(ctx):
    """Commands for tech Users"""
    pass


@tech.group("show")
@click.pass_context
def show(ctx):
    """Display an Egeria Object"""
    pass


@show.command('guid-info')
@click.argument('guid', nargs=1)
@click.pass_context
def show_guid_infos(ctx, guid):
    """Display a live status view of known platforms

    Usage: show guid-info <a guid>

    """
    c = ctx.obj
    display_guid(guid, c.server, c.url,
                 c.userid, c.password, c.jupyter, c.width)


@show.command('tech-types')
@click.option('--search-string', default='*', help='Tech type to search for')
@click.pass_context
def show_tech_types(ctx, search_string):
    """List deployed technology types

    Usage: show tech-types <optional search-string>

    All tech-types will be returned if no search-string is specified.

    """

    c = ctx.obj
    display_tech_types(search_string, c.view_server, c.view_server_url,
                       c.userid, c.password)


@show.command('tech-details')
@click.argument('tech-name')
@click.pass_context
def show_tech_details(ctx, tech_name):
    """Display a live status view of Egeria servers for the specified Egeria platform

    Usage: show tech-details <tech-name>

           tech-name is a valid technology name (see 'show tech-types')
    """
    c = ctx.obj
    tech_details_viewer(tech_name, c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width)


@show.command("asset-types")
@click.pass_context
def show_asset_types(ctx):
    """Display engine-host status information"""
    c = ctx.obj
    display_asset_types(c.view_server, c.view_server_url,
                        c.userid, c.password,
                        c.jupyter, c.width)


@show.command('registered-services')
@click.option('--services',
              type=click.Choice(['all', 'access-services', 'common-services', 'engine-services',
                                 'governance-services', 'integration-services', 'view-services'],
                                case_sensitive=False), default='all', help='Which service group to display')
@click.pass_context
def show_registered_services(ctx, services):
    """Show information about a registered services"""
    c = ctx.obj
    display_registered_svcs(services, c.view_server, c.view_server_url,
                            c.userid, c.password, c.jupyter, c.width)


@show.command('relationship-types')
@click.option('--rel-type', default='AssetOwner', help='Relationship type to get information about')
@click.pass_context
def show_relationship_types(ctx, rel_type):
    """Show information about the specified relationship type"""
    c = ctx.obj
    display_relationship_types(rel_type, c.view_server, c.view_server_url,

                               c.userid, c.password, False, c.jupyter, c.width)


@show.command("tech-templates")
@click.pass_context
@click.option('--search-string', default='*', help='Technology type to get information about')
def tech_templates(ctx, search_string):
    """Display template information about the specified technology."""
    c = ctx.obj
    template_viewer(search_string, c.view_server, c.view_server_url,
                    c.userid, c.password, c.jupyter, c.width)


@show.command("tech-template-spec")
@click.pass_context
@click.option('--search-string', default='*', help='Technology type to get information about')
def tech_template_spec(ctx, search_string):
    """Display template specification information about the specified technology."""
    c = ctx.obj
    display_templates_spec(search_string, c.view_server, c.view_server_url,
                           c.userid, c.password, c.jupyter, c.width)


@show.command("valid-metadata-values")
@click.pass_context
@click.option('--property', default='projectHealth', help='Metadata property to query')
@click.option('--type-name', default='Project', help='Metadata type to query')
def valid_metadata_values(ctx, property, type_name):
    """Display the valid metadata values for a property and type"""
    c = ctx.obj
    display_metadata_values(property, type_name, c.view_server, c.view_server_url,
                            c.userid, c.password, False, c.jupyter, c.width)


#
#  tech Users: Tell
#

@tech.group('tell')
@click.pass_context
def tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


#
#   Catalog User: Show
#

@cli.group('cat')
@click.pass_context
def cat(ctx):
    """Commands for the more tech user"""
    pass


@cat.group("show")
@click.pass_context
def show(ctx):
    """Display an Egeria Object"""
    pass


@show.command('tech-types')
@click.option('--tech_type', default='*', help='Tech type to search for')
@click.pass_context
def show_tech_types(ctx, tech_type):
    """List deployed technology types"""
    c = ctx.obj
    display_tech_types(tech_type, c.view_server, c.view_server_url,
                       c.userid, c.password)


@show.command('tech-type-elements')
@click.option('--tech_type', default='PostgreSQL Server', help='Specific tech type to get elements for')
@click.pass_context
def show_tech_type_elements(ctx, tech_type):
    """List technology type elements"""
    c = ctx.obj
    tech_viewer(tech_type, c.view_server, c.view_server_url,
                c.userid, c.password)


@show.command('tech-type-templates')
@click.option('--tech-type', default='PostgreSQL Server', help='Specific tech type to get elements for')
@click.pass_context
def show_tech_type_templates(ctx, tech_type):
    """List technology type templates"""
    c = ctx.obj
    template_viewer(tech_type, c.view_server, c.view_server_url, c.userid,
                    c.password, c.jupyter, c.width)

@show.command('certification-types')
@click.option('--search-string', default = 'CertificationType', help="")
@click.pass_context
def show_certification_types(ctx, search_string):
    """Show certification types
        - generally stay with the default..
    """
    c = ctx.obj
    display_certifications(search_string, c.view_server, c.view_server_url, c.userid,
                           c.password, c.timeout, c.jupyter, c.width)

@show.command('project-structure')
@click.option('--project', default = 'Clinical Trials Management',
              help="Enter the root project to start from")
@click.pass_context
def show_project_structure(ctx, project):
    """Show the structure of the project starting from a root project"""
    c = ctx.obj
    project_structure_viewer(project, c.view_server, c.view_server_url, c.userid,
                           c.password, c.jupyter, c.width, c.timeout)




@show.command('assets')
@click.argument('search-string')
@click.pass_context
def show_assets(ctx, search_string):
    """Find and display assets

    Usage: show assets <search-string>

           search-string must be greater than four characters.
    """
    c = ctx.obj
    display_assets(search_string, c.view_server, c.view_server_url, c.userid,
                   c.password, 60, c.jupyter, c.width)


@show.command('glossary-terms')
@click.option('--search-string', default='*',
              help='List glossary terms similar to search string - minimum of 4 characters')
@click.option('--glossary_guid', default=None, help='Optionally restrict search to glossary with the specified guid')
@click.pass_context
def show_terms(ctx, search_string, glossary_guid):
    """Find and display glossary terms"""
    c = ctx.obj
    display_glossary_terms(search_string, glossary_guid, c.view_server, c.view_server_url, c.userid,
                           c.password, c.jupyter, c.width)


@show.command('asset-graph')
@click.argument('asset_guid', nargs=1)
@click.pass_context
def show_asset_graph(ctx, asset_guid):
    """Display a tree graph of information about an asset

    Usage: show asset-graph <asset-guid>

           asset-guid must be a valid asset guid. These can be found through other commands such as 'show tech-type-elements'

    """
    c = ctx.obj
    asset_viewer(asset_guid, c.view_server, c.view_server_url, c.userid,
                 c.password, c.jupyter, c.width)


@show.command('collection')
@click.option('--root_collection', default='Root Sustainability Collection',
              help='View of tree of collections from a given root')
@click.pass_context
def show_asset_graph(ctx, root_collection):
    """Display a tree graph of information about an asset """
    c = ctx.obj
    collection_viewer(root_collection, c.view_server, c.view_server_url, c.userid,
                      c.password, c.jupyter, c.width)


@show.command('projects')
@click.option('--search-string', default='*',
              help='List Projects by Search String')
@click.pass_context
def show_projects(ctx, search_string):
    """Display a list of Egeria projects """
    c = ctx.obj
    display_project_list(search_string, c.view_server, c.view_server_url, c.userid,
                         c.password, False, c.jupyter, c.width)


@show.command('to-dos')
@click.option('--search-string', default='*',
              help='View the list of To-Do items')
@click.pass_context
def show_todos(ctx, search_string):
    """Display a tree graph of information about an asset """
    c = ctx.obj
    display_to_dos(search_string, c.view_server, c.view_server_url, c.userid,
                   c.password, c.jupyter, c.width)


#
#  Catalog User: Tell
#

@cat.group('tell')
@click.pass_context
def tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


@tell.group('survey')
@click.pass_context
def survey(ctx):
    """Refresh the specified integration connector or ALL connectors if not specified"""
    c = ctx.obj
    pass


@survey.command('survey-uc-server')
@click.pass_context
@click.option('--uc_endpoint', default='https://localhost:8080',
              help="Endpoint of the Unity Catalog Server to Survey")
def survey_uc_server(ctx, uc_endpoint):
    """Survey the Unity Catalog server at the given endpoint"""
    c = ctx.obj
    pass
    # restart_connector(connector, c.integration_daemon, c.integration_daemon_url,
    #                   c.userid, c.password)


#
#  Operations: Show
#

@cli.group('ops')
@click.pass_context
def ops(ctx):
    """ Commands to understand and manage operations"""
    pass


@ops.group("show")
@click.pass_context
def show(ctx):
    """Display an Egeria Object"""
    pass


@show.group('platforms')
@click.pass_context
def show_platform(ctx):
    """Group of commands to show information about Egeria platforms"""
    pass


@show_platform.command('status')
@click.pass_context
def show_platform_status(ctx):
    """Display a live status view of known platforms"""
    c = ctx.obj
    p_display_status(c.view_server, c.view_server_url,
                     c.admin_user, c.admin_user_password)


@show.group("servers")
@click.pass_context
def show_server(ctx):
    """Group of commands to show information about Egeria servers"""
    pass


@show_server.command('status')
@click.option('--full', is_flag=True, default=False, help='If True, full server descriptions will be shown')
@click.pass_context
def show_server_status(ctx, full):
    """Display a live status view of Egeria servers for the specified Egeria platform"""
    c = ctx.obj
    if full:
        display_list(c.metadata_store, c.metadata_store_url, c.admin_user, c.admin_user_password, c.jupyter, c.width)
    else:
        s_display_status(c.metadata_store, c.metadata_store_url, c.admin_user, c.admin_user_password, c.jupyter,
                         c.width)


@show.group("engines")
@click.pass_context
def engine_host(ctx):
    """Group of commands to show information about Egeria engines"""
    pass


@engine_host.command("status")
@click.option('--list', is_flag=True, default=False, help='If True, a paged list will be shown')
@click.pass_context
def gov_eng_status(ctx, list):
    """Display engine-host status information"""
    c = ctx.obj
    display_gov_eng_status(c.engine_host, c.engine_host_url,
                           c.userid, c.password,
                           list, c.jupyter, c.width)


@engine_host.command('activity')
@click.option('--list', is_flag=True, default=False, help='If True, a paged list will be shown')
@click.pass_context
def eng_activity_status(ctx, list):
    """Show Governance Activity in engine-host"""
    c = ctx.obj
    display_engine_activity(c.view_server, c.view_server_url,
                            c.admin_user, c.admin_user_password,
                            list, c.jupyter, c.width)


@show.group('integrations')
@click.pass_context
def integrations(ctx):
    """Group of commands to show information about Egeria integrations"""
    pass


@integrations.command("status")
@click.option('--list', is_flag=True, default=False, help='If True, a paged list will be shown')
@click.pass_context
def integrations_status(ctx, list):
    """Display integration-daemon status information"""
    c = ctx.obj
    display_integration_daemon_status(c.integration_daemon, c.integration_daemon_url,
                                      c.view_server, c.view_server_url,
                                      c.userid, c.password, list, c.jupyter, c.width)


@integrations.command("targets")
@click.pass_context
@click.argument('connector', nargs=1)
def integrations_status(ctx, connector):
    """Display Catalog Targets for a connector"""
    c = ctx.obj
    display_catalog_targets(connector, c.view_server, c.view_server_url,
                            c.userid, c.password, c.jupyter, c.width)


#
#  Operations: Tell
#

@ops.group('tell')
@click.pass_context
def tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


@tell.group('integration-daemon')
@click.pass_context
def integration_daemon(ctx):
    """Group of commands to an integration-daemon"""
    pass


@integration_daemon.command('refresh')
@click.pass_context
@click.option('--connector', default='all', help="Name of connector to refresh or 'all' to refresh all")
def refresh_connectors(ctx, connector):
    """Refresh the specified integration connector or ALL connectors if not specified"""
    c = ctx.obj
    refresh_connector(connector, c.integration_daemon, c.integration_daemon_url,
                      c.userid, c.password)


@integration_daemon.command('restart')
@click.pass_context
@click.option('--connector', default='all', help="Name of connector to restart or 'all' to restart all")
def restart_connectors(ctx, connector):
    """Restart the specified integration connector or ALL connectors if not specified"""
    c = ctx.obj
    restart_connector(connector, c.integration_daemon, c.integration_daemon_url,
                      c.userid, c.password)


integration_daemon.add_command(add_catalog_target)
integration_daemon.add_command(remove_catalog_target)
integration_daemon.add_command(update_catalog_target)
integration_daemon.add_command(stop_server)
integration_daemon.add_command(start_server)


@tell.group('engine-host')
@click.pass_context
def engine_host(ctx):
    """Group of commands to an engine-host"""
    pass


engine_host.add_command(start_engine_host)
engine_host.add_command(stop_engine_host)

tell.add_command(load_archive)

if __name__ == '__main__':
    cli()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A command line interface for Egeria Users - of individual relevance

This is an emerging capability based on the **click** package. Feedback welcome!

"""
import click
from trogon import tui

from examples.widgets.cli.ops_config import Config

from examples.widgets.my.monitor_open_todos import display_todos
from examples.widgets.my.monitor_my_todos import display_my_todos
from examples.widgets.my.list_my_profile import display_my_profile
from examples.widgets.my.list_my_roles import display_my_roles



# class Config(object):
#     def __init__(self, server: str = None, url: str = None, userid:str = None, password:str = None,
#                  timeout:int = 30, paging: bool = False):
#         self.server = server
#         self.url = url
#         self.userid = userid
#         self.password = password
#         self.timeout = timeout
#         self.paging = paging
#
#
# pass_config = click.make_pass_decorator(Config)

@tui()
# @tui('menu', 'menu', 'A textual command line interface')
@click.version_option("0.0.1", prog_name="egeria_ops")

@click.group()
@click.option('--server', default='active-metadata-store', envvar='EGERIA_METADATA_STORE',
              help='Egeria metadata store to work with')
@click.option('--url', default='https://localhost:9443', envvar='EGERIA_PLATFORM_URL',
              help='URL of Egeria metadata store platform to connect to')
@click.option('--integration-daemon', default='integration-daemon', envvar='EGERIA_INTEGRATION_DAEMON',
              help='Egeria integration daemon to work with')
@click.option('--integration_daemon_url', default='https://localhost:9443', envvar='EGERIA_INTEGRATION_DAEMON_URL',
              help='URL of Egeria integration daemon platform to connect to')
@click.option('--view_server', default='view-server', envvar='EGERIA_VIEW_SERVER',
              help='Egeria view server to work with')
@click.option('--view_server_url', default='https://localhost:9443', envvar='EGERIA_VIEW_SERVER_URL',
              help='URL of Egeria view server platform to connect to')
@click.option('--engine_host', default='engine-host', envvar='EGERIA_ENGINE_HOST',
              help='Egeria engine host to work with')
@click.option('--engine_host_url', default='https://localhost:9443', envvar='EGERIA_ENGINE_HOST_URL',
              help='URL of Egeria engine host platform to connect to')
@click.option('--admin_user', default='garygeeke', envvar='EGERIA_ADMIN_USER', help='Egeria admin user')
@click.option('--admin_user_password', default='secret', envvar='EGERIA_ADMIN_PASSWORD',
              help='Egeria admin password')
@click.option('--userid', default='erinoverview', envvar='EGERIA_USER', help='Egeria user')
@click.option('--password', default='secret', envvar='EGERIA_PASSWORD',
              help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.option('--verbose', is_flag=True, default=False, help='Enable verbose mode')
@click.option('--paging', is_flag=True, default=False, help='Enable paging snapshots vs live updates')
@click.option('--jupyter', is_flag=True, default=False, envvar='EGERIA_JUPYTER',
              help='Enable for rendering in a Jupyter terminal')
@click.option('--width', default=200, envvar='EGERIA_WIDTH', help='Screen width, in characters, to use')
@click.pass_context
def cli(ctx, server, url, view_server, view_server_url, integration_daemon, integration_daemon_url,
        engine_host, engine_host_url, admin_user, admin_user_password, userid, password, timeout, paging,
        verbose, jupyter, width):
    """An Egeria Command Line interface for Operations """
    ctx.obj = Config(server, url, view_server, view_server_url, integration_daemon,
                     integration_daemon_url, engine_host, engine_host_url,
                     admin_user, admin_user_password, userid, password,
                     timeout, paging, verbose, jupyter, width)
    ctx.max_content_width = 200
    ctx.ensure_object(Config)
    if verbose:
        click.echo(f"we are in verbose mode - server is {server}")


@cli.group("show")
@click.pass_context
def show(ctx):
    """Display an Egeria Object"""
    pass


@show.command('my-profile')
@click.pass_context
def show_my_profile(ctx):
    """Display my profiles

    Usage: show my-profile

    """
    c = ctx.obj
    display_my_profile( c.view_server, c.view_server_url,
                 c.userid, c.password, c.jupyter, c.width)

@show.command('my-roles')
@click.pass_context
def show_my_roles(ctx):
    """Display my profiles

    Usage: show my-profile

    """
    c = ctx.obj
    display_my_roles( c.view_server, c.view_server_url,
                 c.userid, c.password, c.jupyter, c.width)


@show.command('my-to-dos')
@click.pass_context
def show_my_todos(ctx):
    """Show my To-Dos

    Usage: show my-to-dos

    """
    c = ctx.obj
    display_my_todos(c.view_server, c.view_server_url,
                       c.userid, c.password, c.jupyter, c.width)


@show.command('open-to-dos')
@click.pass_context
def show_open_todos(ctx):
    """Display a live status view of Egeria servers for the specified Egeria platform

    Usage: show tech-details <tech-name>

           tech-name is a valid technology name (see 'show tech-types')
    """
    c = ctx.obj
    display_todos( c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width)

#
#  Tell
#

@cli.group('tell')
@click.pass_context
def tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


if __name__ == '__main__':
    cli()



#!/usr/bin/env python3
"""
SPDX-Lic
ense-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple display for glossary terms
"""
import os
import argparse
import time

from rich import box
from rich.console import Console
from rich.prompt import Prompt
from rich.table import Table

from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)
from pyegeria.my_profile_omvs import MyProfile

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get('EGERIA_INTEGRATION_DAEMON_URL', 'https://localhost:9443')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')
EGERIA_JUPYTER = bool(os.environ.get('EGERIA_JUPYTER', 'False'))
EGERIA_WIDTH = int(os.environ.get('EGERIA_WIDTH', '200'))

def display_to_dos(search_string: str, server: str, url: str, username: str, user_pass:str,
                   jupyter:bool=EGERIA_JUPYTER, width:int = EGERIA_WIDTH):

    m_client = MyProfile(server, url, user_id=username)
    token = m_client.create_egeria_bearer_token(username, user_pass)

    def generate_table(search_string:str = '*') -> Table:
        """Make a new table."""
        table = Table(
            title=f"Open ToDos for Platform {url} @ {time.asctime()}",
            # style = "black on grey66",
            header_style="white on dark_blue",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"ToDos for Server '{server}' @ Platform - {url}",
            expand=True
        )

        table.add_column("Name")
        table.add_column("Type Name")

        table.add_column("Created")
        table.add_column("Priority")
        table.add_column("Due")
        table.add_column("Completion")
        table.add_column("Status")
        table.add_column("Sponsor")

        todo_items = m_client.find_to_do(search_string)

        if type(todo_items) is str:
            name = " "
            type_name = " "
            created = " "
            priority = " "
            due = " "
            completed = " "

            status = " "
            sponsor = " "
        else:
            for item in todo_items:
                props = item["properties"]
                name = props["name"]
                type_name = props.get("toDoType", " ")
                created = props.get("creationTime", " ")
                priority = str(props.get("priority", " "))
                due = props.get("dueTime", " ")
                completed = props.get("completionTime", " ")
                status = props.get("status")
                # assigned_actors = item["assignedActors"]
                # sponsor = assigned_actors[0].get("uniqueName", " ")
                sponsor = "erinoverview"
                if status in ("WAITING", "OPEN"):
                    status = f"[yellow]{status}"
                elif status in ("IN_PROGRESS", "COMPLETE"):
                    status = f"[green]{status}"
                else:
                    status = f"[red]{status}"

                table.add_row(
                    name, type_name, created, priority, due, completed, status, sponsor
                )

        m_client.close_session()
        return table

    try:
        # with Live(generate_table(), refresh_per_second=4, screen=True) as live:
        #     while True:
        #         time.sleep(2)
        #         live.update(generate_table())
        console = Console(width=width, force_terminal=not jupyter)
        with console.pager():

            console.print(generate_table(search_string))


    except (InvalidParameterException, PropertyServerException, UserNotAuthorizedException) as e:
        print_exception_response(e)
    except KeyboardInterrupt:
        pass
    finally:
        m_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    # parser.add_argument("--sponsor", help="Name of sponsor to search")
    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    try:
        search_string = Prompt.ask("Enter the ToDo you are searching for:", default="*")
        display_to_dos(search_string, server, url, userid, user_pass)
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()